@@ -1,4 +1,4 @@
-/*      $Id: misc.c,v 1.47 2004/09/23 06:26:50 olivier Exp $
+/*      $Id: misc.c 12192 2004-12-09 22:44:20Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -101,17 +101,20 @@ createGC (ScreenInfo *screen_info, char *col, int func, XFontStruct * font,
 }
 
 void
-sendClientMessage (ScreenInfo *screen_info, Window w, Atom a, Atom x, Time timestamp)
+sendClientMessage (ScreenInfo *screen_info, Window w, int atom_id, Time timestamp)
 {
+    DisplayInfo *display_info = NULL;
     XClientMessageEvent ev;
 
+    g_return_if_fail ((atom_id > 0) && (atom_id < NB_ATOMS));
     TRACE ("entering sendClientMessage");
 
+    display_info = screen_info->display_info;
     ev.type = ClientMessage;
     ev.window = w;
-    ev.message_type = a;
+    ev.message_type = display_info->atoms[WM_PROTOCOLS];
     ev.format = 32;
-    ev.data.l[0] = x;
+    ev.data.l[0] = display_info->atoms[atom_id];
     ev.data.l[1] = timestamp;
     XSendEvent (myScreenGetXDisplay (screen_info), w, FALSE, 0L, (XEvent *)&ev);
 }
@@ -182,3 +185,69 @@ placeSidewalks(ScreenInfo *screen_info, gboolean activate)
                         gdk_screen_get_width (screen_info->gscr), 1, FALSE);
     }
 }
+
+/* 
+   gdk_screen_get_monitor_at_point () doesn't give accurate results
+   when the point is off screen, use my own implementation from xfce 3
+ */
+gint 
+find_monitor_at_point (GdkScreen *screen, gint x, gint y)
+{
+    static gint cache_monitor = -1;
+    static gint cache_x;
+    static gint cache_y;
+
+    gint dx, dy;
+    gint center_x, center_y;
+    guint32 distsquare, min_distsquare;
+
+    gint num_monitors, i;
+    gint nearest_monitor;
+    GdkRectangle monitor;
+
+    g_return_val_if_fail (GDK_IS_SCREEN (screen), -1);
+
+    /* Cache system */
+    if ((cache_monitor >= 0) && (x == cache_x) && (y == cache_y))
+    {
+        return (cache_monitor);
+    }
+    
+    cache_x = x;
+    cache_y = y;
+
+    /* No monitor has been eligible, use the closest one */
+
+    min_distsquare = ((guint32) 0xffffffff);
+    nearest_monitor = 0;
+
+    num_monitors = gdk_screen_get_n_monitors (screen);
+    for (i = 0; i < num_monitors; i++)
+    {
+        gdk_screen_get_monitor_geometry (screen, i, &monitor);
+
+        if ((x >= monitor.x) && (x < monitor.x + monitor.width) &&
+            (y >= monitor.y) && (y < (monitor.y + monitor.height)))
+        {
+            cache_monitor = i;
+            return i;
+        }
+
+        center_x = monitor.x + (monitor.width / 2);
+        center_y = monitor.y + (monitor.height / 2);
+
+        dx = x - center_x;
+        dy = y - center_y;
+
+        distsquare = (dx * dx) + (dy * dy);
+
+        if (distsquare < min_distsquare)
+        {
+            min_distsquare = distsquare;
+            nearest_monitor = i;
+        }
+    }
+
+    cache_monitor = nearest_monitor;
+    return (nearest_monitor);
+}
